package player;

import java.io.File;

/**
 * The class FileTypes defines which file types can be loaded by MP3Player.
 * @author Martin Henke
 *
 */
class FileTypes {
	private static final String MP3 = ".mp3";
	private static final String WAV =".wav";
	
	/**
	 * Gets the Description string for the JFileChooser dialog.
	 * @return The Description string for the JFileChooser dialog. 
	 */
	public static String getFileFilterDescription(){
		return String.format("Sound Files (*%1$s; *%2$s)",MP3, WAV);
	}
	
	/**
	 * Checks if the file is supported.
	 * @param file The file to check.
	 * @return True if the file is supported, otherwise false.
	 */
	public static boolean isSupported(File file){
		if (file != null){
			if (file.isDirectory()){
				return true;
			} else { 
				return isSupportedFileExtension(file);
			}
		} else {
			return false;
		}
	}
	
	/**
	 * Checks if the file has a supported file extension.
	 * @param file The file to check.
	 * @return True if the file extension is supported, otherwise false.
	 */
	public static boolean isSupportedFileExtension(File file){
		String fileName;
		if (file != null){
			if (file.isDirectory()){
				return false;
			} else {
				fileName = file.getName().toLowerCase();
				if (fileName.endsWith(MP3)){
					return true;
				} else if (fileName.endsWith(WAV)){
					return true;
				} else {
					return false;
				}
			}
		} else {
			return false;
		}
	}
}
