package player;

import java.awt.Color;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.Font;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

/**
 * Starts the application and provides methods for Menu skinning.
 * @author Martin Henke
 *
 */
public class MP3Player {

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					MainDialog frame = new MainDialog();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
	
	/**
	 * Sets background, foreground and font to a JMenuBar and child menus.
	 * @param bar The JMenuBar to process.
	 * @param backColor The background to use for the JMenuBar and all child menus.
	 * @param foreColor The foreground to use for the JMenuBar and all child menus.
	 * @param font The font to use for the JMenuBar and all child menus.
	 */
	static void setSkinMenu(JMenuBar bar, Color backColor, Color foreColor, Font font){
		bar.setBackground(backColor);
		for (int i = 0; i < bar.getMenuCount(); i++){
			setSkinMenu(bar.getMenu(i), backColor, foreColor, font);
		}
	}
	
	/**
	 * Sets background, foreground and font to a JPopupMenu and child menus.
	 * @param menu The JPopupMenu to process.
	 * @param backColor The background to use for the JPopupMenu and all child menus.
	 * @param foreColor The foreground to use for the JPopupMenu and all child menus.
	 * @param font The font to use for the JPopupMenu and all child menus.
	 */
	static void setSkinMenu(JPopupMenu menu, Color backColor, Color foreColor, Font font){
		Component c;
		
		menu.setBackground(backColor);
		menu.setForeground(foreColor);
		menu.setFont(font);

		for (int i = 0; i < menu.getComponentCount(); i++){
			c = menu.getComponent(i);
			if (c instanceof JMenu){
				setSkinMenu((JMenu)c, backColor, foreColor, font);
			}
		}
	}
	
	/**
	 * Sets background, foreground and font to a JMenu and child menus.
	 * @param menu The JMenu to process.
	 * @param backColor The background to use for the JMenu and all child menus.
	 * @param foreColor The foreground to use for the JMenu and all child menus.
	 * @param font The font to use for the JMenu and all child menus.
	 */
	static void setSkinMenu(JMenu menu, Color backColor, Color foreColor, Font font){
		JMenuItem item;
		menu.setBackground(backColor);
		menu.setForeground(foreColor);
		menu.setFont(font);

		for (int i = 0; i < menu.getItemCount(); i++){
			item = menu.getItem(i);
			if (item instanceof JMenu){
				setSkinMenu((JMenu)item, backColor, foreColor, font);
				item.getParent().setBackground(backColor);	// This is required to set the background for submenus.
			} else {
				setSkinMenu(item, backColor, foreColor, font);
			}
		}		
	}
	
	/**
	 * Sets background, foreground and font to a JMenuItem.
	 * @param item The JMenuItem to process.
	 * @param backColor The background to use for the JMenuItem.
	 * @param foreColor The foreground to use for the JMenuItem.
	 * @param font The font to use for the JMenuItem.
	 */
	static void setSkinMenu(JMenuItem item, Color backColor, Color foreColor, Font font){
		item.setBackground(backColor);
		item.setForeground(foreColor);
		item.setFont(font);
	}
}
