package player;

import java.awt.*;
import java.awt.Dimension;
import java.awt.Point;
import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.text.DecimalFormat;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.FileFilter;
import javax.xml.bind.JAXBException;

import configuration.*;
import configuration.Button;

import javafx.scene.media.*;
import javafx.util.Duration;
import javafx.embed.swing.JFXPanel;
import generic.BackgroundPanel;
import generic.BackgroundPanel.Style;
import generic.ImageLabel;
import generic.SkinLoadDialog;

import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.awt.event.KeyListener;

/**
 * The class MainDialog inherits from JFrame and provides the basic functions for MP3Player.
 * @author Martin Henke
 *
 */
public class MainDialog extends JFrame implements ActionListener, MouseListener, WindowListener, KeyListener {
	private static final long serialVersionUID = 8191456474747236778L;
	
	private JPanel contentPane;
	private MediaPlayer mediaPlayer = null;
	private JFileChooser mediaFileChooser;
	private Skin skin = null;
	private ImageLabel lblPlay;
	private ImageLabel lblPause;
	private ImageLabel lblStop;
	private ImageLabel lblFastForward;
	private ImageLabel lblRewind;
	private ImageLabel lblSkip;
	private ImageLabel lblRestart;
	private ImageLabel lblPlayList;
	private ImageLabel lblTitle;
	private ImageLabel lblElapsedTime;
	private ClickHelper chPlay = new ClickHelper(null);
	private ClickHelper chPause = new ClickHelper(null);
	private ClickHelper chStop = new ClickHelper(null);
	private ClickHelper chFastForward = new ClickHelper(null);
	private ClickHelper chRewind = new ClickHelper(null);
	private ClickHelper chSkip = new ClickHelper(null);
	private ClickHelper chRestart = new ClickHelper(null);
	private ClickHelper chPlayList = new ClickHelper(null);
	private DecimalFormat df = new DecimalFormat("00");
	private PlayListWindow playList;
	
	private BackgroundPanel backgroundPanel;
	private JMenuBar menuBar;
	private JMenu mnFile;
	private JMenuItem mntmOpen;
	private JMenuItem mntmExit;
	private JMenu mnPreferences;
	private JMenuItem mntmOpenSkin;
	private final Timer timElapsed = new Timer(0, (ActionListener) null);
	private JMenu mnHelp;
	private JMenuItem mntmInfo;
	private JMenu mnPlayList;
	private JMenuItem mntmPlayListNew;
	private JMenuItem mntmPlayListSave;
	private JMenuItem mntmPlayListSaveAs;
	private JMenuItem mntmPlayListOpen;
	
	/**
	 * Create the frame.
	 * @throws URISyntaxException 
	 */
	public MainDialog() throws URISyntaxException {
		addKeyListener(this);
		addWindowListener(this);
		timElapsed.addActionListener(this); 
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 450, 300);
		
		menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		mnFile = new JMenu("File");
		mnFile.setMnemonic('f');
		menuBar.add(mnFile);
		
		mntmOpen = new JMenuItem("Open...");
		mntmOpen.addActionListener(this);
		mntmOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		mntmOpen.setMnemonic(KeyEvent.VK_O);
		mnFile.add(mntmOpen);
		
		mntmExit = new JMenuItem("Exit");
		mntmExit.addActionListener(this);
		
		mnPlayList = new JMenu("Play List");
		mnPlayList.setMnemonic(KeyEvent.VK_P);
		mnFile.add(mnPlayList);
		
		mntmPlayListNew = new JMenuItem("New");
		mntmPlayListNew.addActionListener(this);
		mnPlayList.add(mntmPlayListNew);
		
		mntmPlayListSave = new JMenuItem("Save");
		mntmPlayListSave.addActionListener(this);
		
		mntmPlayListOpen = new JMenuItem("Open");
		mntmPlayListOpen.addActionListener(this);
		mnPlayList.add(mntmPlayListOpen);
		mnPlayList.add(mntmPlayListSave);
		
		mntmPlayListSaveAs = new JMenuItem("Save As...");
		mntmPlayListSaveAs.addActionListener(this);
		mnPlayList.add(mntmPlayListSaveAs);
		mntmExit.setMnemonic(KeyEvent.VK_X);
		mntmExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, InputEvent.ALT_MASK));
		mnFile.add(mntmExit);
		
		mnPreferences = new JMenu("Preferences");
		mnPreferences.setMnemonic('p');
		menuBar.add(mnPreferences);
		
		mntmOpenSkin = new JMenuItem("Open Skin...");
		mntmOpenSkin.setMnemonic('o');
		mntmOpenSkin.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
		mntmOpenSkin.addActionListener(this);
		mnPreferences.add(mntmOpenSkin);
		
		mnHelp = new JMenu("Help");
		mnHelp.setMnemonic('h');
		menuBar.add(mnHelp);
		
		mntmInfo = new JMenuItem("Info");
		mntmInfo.addActionListener(this);
		mntmInfo.setMnemonic(KeyEvent.VK_I);
		mnHelp.add(mntmInfo);
		contentPane = new JPanel();
		contentPane.setAlignmentX(0.0f);
		contentPane.setAlignmentY(0.0f);
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		contentPane.setLayout(null);
		
		backgroundPanel = new BackgroundPanel();
		contentPane.add(backgroundPanel);
		backgroundPanel.setLayout(null);
		backgroundPanel.setLocation(0, 0);
		backgroundPanel.setSize(new Dimension(1, 1));
		
		lblPlay = new ImageLabel("play");
		lblPlay.addMouseListener(this);
		lblPlay.setBounds(10, 11, 46, 14);
		backgroundPanel.add(lblPlay);
		
		lblPause = new ImageLabel("pause");
		lblPause.addMouseListener(this);
		lblPause.setBounds(66, 11, 46, 14);
		backgroundPanel.add(lblPause);
		
		lblFastForward = new ImageLabel("fast forward");
		lblFastForward.addMouseListener(this);
		lblFastForward.setBounds(147, 11, 46, 14);
		backgroundPanel.add(lblFastForward);
		
		lblRewind = new ImageLabel("rewind");
		lblRewind.addMouseListener(this);
		lblRewind.setBounds(219, 11, 46, 14);
		backgroundPanel.add(lblRewind);
		
		lblSkip = new ImageLabel("skip");
		lblSkip.addMouseListener(this);
		lblSkip.setBounds(264, 11, 46, 14);
		backgroundPanel.add(lblSkip);
		
		lblRestart = new ImageLabel("restart");
		lblRestart.addMouseListener(this);
		lblRestart.setBounds(20, 36, 46, 14);
		backgroundPanel.add(lblRestart);
		
		lblPlayList = new ImageLabel("playList");
		lblPlayList.addMouseListener(this);
		lblPlayList.setBounds(76, 36, 46, 14);
		backgroundPanel.add(lblPlayList);
		
		lblTitle = new ImageLabel();
		lblTitle.setBounds(147, 36, 46, 14);
		backgroundPanel.add(lblTitle);
		
		lblElapsedTime = new ImageLabel("elapsed time");
		lblElapsedTime.setText("00:00");
		lblElapsedTime.setBounds(219, 36, 46, 14);
		backgroundPanel.add(lblElapsedTime);
		
		lblStop = new ImageLabel("stop");
		lblStop.addMouseListener(this);
		lblStop.setBounds(30, 61, 46, 14);
		backgroundPanel.add(lblStop);
		
		mediaFileChooser = new JFileChooser();
		mediaFileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		mediaFileChooser.setMultiSelectionEnabled(true);
		mediaFileChooser.addChoosableFileFilter(new FileFilter(){
			@Override
			public boolean accept(File f) {
				return FileTypes.isSupported(f);
			}

			@Override
			public String getDescription() {
				return FileTypes.getFileFilterDescription();
			}			
		});
		
		activateToolkit();
	}
	
	/**
	 * Activates the JFX toolit by creating a JFXPanel.
	 */
	private void activateToolkit(){
		new JFXPanel();
	}

	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == mntmPlayListOpen) {
			mntmPlayListOpenActionPerformed(e);
		} else if (e.getSource() == mntmPlayListNew) {
			mntmPlayListNewActionPerformed(e);
		} else if (e.getSource() == mntmPlayListSaveAs) {
			mntmPlayListSaveAsActionPerformed(e);
		} else if (e.getSource() == mntmPlayListSave) {
			mntmPlayListSaveActionPerformed(e);
		} else if (e.getSource() == mntmInfo) {
			mntmInfoActionPerformed(e);
		} else if (e.getSource() == mntmOpen) {
			mntmOpenActionPerformed(e);
		} else if (e.getSource() == mntmExit) {
			mntmExitActionPerformed(e);
		} else if (e.getSource() == mntmOpenSkin) {
			mntmOpenSkinActionPerformed(e);
		} else if (e.getSource() == timElapsed){
			timElapsedActionPerformed(e);
		}
	}
	
	/**
	 * Updates the elapsed time display every 500 ms.
	 * @param e The associated ActionEvent object.
	 */
	protected void timElapsedActionPerformed(ActionEvent e){
		Duration currentTime = mediaPlayer.getCurrentTime();
		long milliseconds = (long)currentTime.toMillis();
		long seconds = milliseconds / 1000;
		long minutes = seconds / 60;
		seconds = seconds - (minutes * 60);
		lblElapsedTime.setText(String.format("%1$s:%2$s", df.format(minutes), df.format(seconds)));
		if (mediaPlayer != null && mediaPlayer.getTotalDuration().toMillis() < currentTime.toMillis()){
				playList.playNextFile();
		}
	}

	/**
	 * Exits the program. 
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmExitActionPerformed(ActionEvent e) {
		System.exit(0);
	}
	
	/**
	 * Adds files to the play list.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmOpenActionPerformed(ActionEvent e) {
		addMedia();
	}

	/**
	 * Adds files to the play list.
	 */
	public void addMedia(){
		if (mediaFileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION){
			File[] selectedFiles = mediaFileChooser.getSelectedFiles();
			//int index = playList.getEntryCount();	// Required if the first added file shoulc be played.
			playList.addFiles(selectedFiles);
			//playList.playFile(index);	// Play the first added file.
		}		
	}
	
	/**
	 * Loads a file into the MediaPlayer and plays it.
	 * @param mediaFile The file to load.
	 */
	public void loadMediaFile(File mediaFile){
		if (mediaPlayer != null){
			mediaPlayer.stop();
		}
		Media media = new Media(mediaFile.toURI().toString());
		mediaPlayer = new MediaPlayer(media);

		mediaPlayer.play();
		this.timElapsed.start();
		this.lblTitle.setText(mediaFile.getName());
	}
	
	/**
	 * Opens a skin and changes the player style accordingly. The window position is not changed. 
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmOpenSkinActionPerformed(ActionEvent e) {
		SkinLoadDialog skinLoadDialog = new SkinLoadDialog();
		if (skinLoadDialog.showDialog() == JFileChooser.APPROVE_OPTION){
			loadSkin(skinLoadDialog.getSelectedFile(), false);
		}
	}

	/**
	 * Loads a skin and sets the window location of the window.
	 * @param name The skin to load.
	 * @param setWindowLocation Defines if the window location should be changed.
	 */
	private void loadSkin(String name, boolean setWindowLocation){
		try {
			skin = Skin.load(name);
			setSkin(setWindowLocation);
			playList.setSkin(skin, setWindowLocation, this.getBounds());
		} catch (ClassNotFoundException e) {
			JOptionPane.showMessageDialog(this, "Unable to read skin. The file may be corrupt.", "Skin Error", JOptionPane.WARNING_MESSAGE);			
			e.printStackTrace();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(this, "Unable to read skin. The file may be missing or locked.", "Skin Error", JOptionPane.WARNING_MESSAGE);			
			e.printStackTrace();
		} catch (JAXBException e) {
			JOptionPane.showMessageDialog(this, "Unable to read skin. The file may be corrupt.", "Skin Error", JOptionPane.WARNING_MESSAGE);			
			e.printStackTrace();
		}
	}
	
	/**
	 * Sets the skin settings to the controls.
	 * @param setWindowLocation Defines if the window location should be changed.
	 */
	private void setSkin(boolean setWindowLocation){
		setSkinWindow(setWindowLocation);
		setSkinLabel(this.lblElapsedTime, skin.getElapsedTime());	// No clicks will be processed for this label
		setSkinLabel(this.lblTitle, skin.getTitle());				// No clicks will be processed for this label
		this.chFastForward = setSkinLabel(this.lblFastForward, skin.getFastForward());
		this.chPause = setSkinLabel(this.lblPause, skin.getPause());
		this.chStop = setSkinLabel(this.lblStop, skin.getStop());
		this.chPlay = setSkinLabel(this.lblPlay, skin.getPlay());
		this.chRestart = setSkinLabel(this.lblRestart, skin.getRestart());
		this.chRewind = setSkinLabel(this.lblRewind, skin.getRewind());
		this.chSkip = setSkinLabel(this.lblSkip, skin.getSkip());
		this.chPlayList = setSkinLabel(this.lblPlayList, skin.getPlayList());		
		this.contentPane.setBounds(new Rectangle(new Point(0, 0), this.getSize()));
		this.backgroundPanel.setBounds(this.contentPane.getBounds());
	}
	/**
	 * Set the window related skin settings.
	 * @param setWindowLocation Defines if the window location should be changed.
	 */
	private void setSkinWindow(boolean setWindowLocation){
		ImageControl window = skin.getWindow();
		Control menu = skin.getMenu();
		Point windowLocation = window.getLocation().toPoint();
		if (setWindowLocation){
			if (windowLocation.x == -1 && windowLocation.y == -1){						// Center on the screen in x and y direction.
				this.setLocationRelativeTo(null);
			} else if (windowLocation.x == -1){											// Center on the screen in x direction.
				this.setLocationRelativeTo(null);
				this.setLocation(new Point(this.getLocation().x, windowLocation.y));
			} else if (windowLocation.y == -1){											// Center on the screen in y direction.
				this.setLocationRelativeTo(null);				
				this.setLocation(new Point(windowLocation.x, this.getLocation().y));
			} else {																	// Set window position.
				this.setLocation(window.getLocation().toPoint());
			}
		}
		this.setSize(new Dimension(526, 350));	// This is a setting by WindowBuilder. Do not remove.
		
		// Calculate the size of the window.
		Dimension windowDimension = window.getSize().toDimension();
		Insets insets = this.getInsets();
		windowDimension = new Dimension(windowDimension.width + insets.left + insets.right, windowDimension.height + insets.top + insets.bottom + this.menuBar.getHeight());
		this.setSize(windowDimension);		
		this.contentPane.setBounds(new Rectangle(new Point(0, 0), windowDimension));
		this.backgroundPanel.setBounds(this.contentPane.getBounds());
		this.setTitle(window.getText());
		this.backgroundPanel.setBackground(window.getBackColor().toColor());
		
		// Load background image and style if defined.
		if (window.getImage() != null){
			this.backgroundPanel.setStyle(window.getImage().getStyle());
			this.backgroundPanel.setImage(window.getImage().getImage());
		} else {
			this.backgroundPanel.setImage(null);
			this.backgroundPanel.setStyle(Style.Actual);
		}
		
		// Skin the menus
		MP3Player.setSkinMenu(this.menuBar, menu.getBackColor().toColor(), menu.getForeColor().toColor(), menu.getFont().toFont());
	}
	
	/**
	 * Skin ImageLabel with Control settings.
	 * @param label The label to skin.
	 * @param control The settings to use.
	 */
	private void setSkinLabel(ImageLabel label, Control control){
		label.setLocation(control.getLocation().toPoint());
		label.setSize(control.getSize().toDimension());
		label.setPreferredSize(label.getSize());
		label.setMaximumSize(label.getSize());
		label.setMinimumSize(label.getSize());
		label.setBackground(control.getBackColor().toColor());
		label.setForeground(control.getForeColor().toColor());
		label.setFont(control.getFont().toFont());
		label.setVisible(control.isVisible());
	}
	
	/**
	 * Skin ImageLabel with TextControl settings.
	 * @param label The label to skin.
	 * @param textControl The settings to use.
	 */
	private void setSkinLabel(ImageLabel label, TextControl textControl){
		label.setLocation(textControl.getLocation().toPoint());
		label.setFont(textControl.getFont().toFont());
		label.setText(textControl.getText());
		label.setSize(textControl.getSize().toDimension());
		label.setBackground(textControl.getBackColor().toColor());
		label.setForeground(textControl.getForeColor().toColor());		
		label.setVisible(textControl.isVisible());		
	}
	
	/**
	 * Skin ImageLabel with ImageControl settings.
	 * @param label The label to skin.
	 * @param imageControl The settings to use.
	 */
	private void setSkinLabel(ImageLabel label, ImageControl imageControl){
		label.setLocation(imageControl.getLocation().toPoint());
		label.setSize(imageControl.getSize().toDimension());
		label.setFont(imageControl.getFont().toFont());
		label.setText(imageControl.getText());	
		label.setBackground(imageControl.getBackColor().toColor());
		label.setForeground(imageControl.getForeColor().toColor());
		label.setImage(imageControl.getImage());
		label.setVisible(imageControl.isVisible());
	}
	
	/**
	 * Skin ImageLabel with Button settings.
	 * @param label The label to skin.
	 * @param button The settings to use.
	 */
	private ClickHelper setSkinLabel(ImageLabel label, Button button){
		label.setFont(button.getFont().toFont());
		label.setText(button.getText());
		label.setBackground(button.getBackColor().toColor());
		label.setForeground(button.getForeColor().toColor());
		label.setImage(button.getImage());
		label.setLocation(button.getLocation().toPoint());
		label.setSize(button.getSize().toDimension());
		label.setVisible(button.isVisible());
		return new ClickHelper(button.getMask());
	}
	
	/**
	 * Handles mouse clicks.
	 * @param The associated MouseEvent object.
	 */
	public void mouseClicked(MouseEvent e) {
		if (e.getSource() == lblStop) {
			lblStopMouseClicked(e);
		} else if (e.getSource() == lblRestart) {
			lblRestartMouseClicked(e);
		} else if (e.getSource() == lblPlayList) {
			lblPlayListMouseClicked(e);
		} else if (e.getSource() == lblRewind) {
			lblRewindMouseClicked(e);
		} else if (e.getSource() == lblPause) {
			lblPauseMouseClicked(e);
		} else if (e.getSource() == lblFastForward) {
			lblFastForwardMouseClicked(e);
		} else if (e.getSource() == lblSkip) {
			lblSkipMouseClicked(e);
		} else if (e.getSource() == lblPlay) {
			lblPlayMouseClicked(e);
		}
	}
	/**
	 * Currently not used.
	 */
	public void mouseEntered(MouseEvent e) { }
	/**
	 * Currently not used.
	 */
	public void mouseExited(MouseEvent e) { }
	/**
	 * Currently not used.
	 */
	public void mousePressed(MouseEvent e) { }
	/**
	 * Currently not used.
	 */
	public void mouseReleased(MouseEvent e) { }

	/**
	 * Starts playing the current file or the next file if no file is selected. 
	 * @param The associated MouseEvent object.
	 */
	protected void lblPlayMouseClicked(MouseEvent e) {
		if (mediaPlayer != null && this.chPlay.isClick(e.getPoint())){
			mediaPlayer.play();
			this.timElapsed.start();
		} else {
			playList.playNextFile();
		}
	}
	/**
	 * Plays the next file. 
	 * @param The associated MouseEvent object.
	 */
	protected void lblSkipMouseClicked(MouseEvent e) {
		if (chSkip.isClick(e.getPoint())){
			playList.playNextFile();
		}
	}
	
	/**
	 * Skips the 5 seconds in the file. 
	 * @param The associated MouseEvent object.
	 */
	protected void lblFastForwardMouseClicked(MouseEvent e) {
		if (mediaPlayer != null && this.chFastForward.isClick(e.getPoint())){
			
			mediaPlayer.seek(new Duration(mediaPlayer.getCurrentTime().toMillis() + 5000));
		}
	}

	/**
	 * Pauses playback.
	 * @param The associated MouseEvent object.
	 */
	protected void lblPauseMouseClicked(MouseEvent e) {
		if (mediaPlayer != null && this.chPause.isClick(e.getPoint())){
			mediaPlayer.pause();
		}
	}
	
	/**
	 * Rewinds the current file 5 seconds. 
	 * @param The associated MouseEvent object.
	 */
	protected void lblRewindMouseClicked(MouseEvent e) {
		if (mediaPlayer != null && this.chRewind.isClick(e.getPoint())){
			double currentMillis = mediaPlayer.getCurrentTime().toMillis() - 5000;
			if (currentMillis < 0){
				currentMillis = 0;
			}
			mediaPlayer.seek(new Duration(currentMillis));
		}
	}
	/**
	 * Opens or closes the play list. 
	 * @param The associated MouseEvent object.
	 */
	protected void lblPlayListMouseClicked(MouseEvent e) {
		if (this.chPlayList.isClick(e.getPoint())){
			playList.setVisible(!playList.isVisible());
		}
	}
	
	/**
	 * Restarts the current file. 
	 * @param The associated MouseEvent object.
	 */
	protected void lblRestartMouseClicked(MouseEvent e) {
		if (mediaPlayer != null && this.chRestart.isClick(e.getPoint())){
			mediaPlayer.seek(new Duration(0));
		}
	}
	/**
	 * Stops the current file. 
	 * @param The associated MouseEvent object.
	 */
	protected void lblStopMouseClicked(MouseEvent e) {
		if (mediaPlayer != null && this.chStop.isClick(e.getPoint())){
			mediaPlayer.stop();
			this.timElapsed.stop();
		}
	}
	/**
	 * Currently not used.
	 */
	public void windowActivated(WindowEvent e) { }
	/**
	 * Currently not used.
	 */
	public void windowClosed(WindowEvent e) { }
	/**
	 * Currently not used.
	 */
	public void windowClosing(WindowEvent e) { }
	/**
	 * Currently not used.
	 */
	public void windowDeactivated(WindowEvent e) { }
	/**
	 * Currently not used.
	 */
	public void windowDeiconified(WindowEvent e) { }
	/**
	 * Currently not used.
	 */
	public void windowIconified(WindowEvent e) { }
	/**
	 * Calls thisWindowOpened(e). 
	 * @param e The associated WindowEvent object.
	 */
	public void windowOpened(WindowEvent e) {
		if (e.getSource() == this) {
			thisWindowOpened(e);
		}
	}
	/**
	 * Initializes the play list and the skin. 
	 * @param e The associated WindowEvent object.
	 */
	protected void thisWindowOpened(WindowEvent e) {
		playList = new PlayListWindow(this);
		loadSkin("default", true);
		playList.setVisible(true);
		this.timElapsed.setDelay(500);		
	}

	/**
	 * Opens the info dialog.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmInfoActionPerformed(ActionEvent e) {
		Info info = new Info();
		info.setModal(true);
		info.setVisible(true);
	}
	/**
	 * Currently not used.
	 */
	public void keyPressed(KeyEvent e) { }
	/**
	 * Currently not used.
	 */
	public void keyReleased(KeyEvent e) { }
	/**
	 * Calls thisKeyTyped(e).
	 * @param e The associated KeyEvent object.
	 */
	public void keyTyped(KeyEvent e) {
		if (e.getSource() == this) {
			thisKeyTyped(e);
		}
	}
	/**
	 * Checks if one of the top menus has been selected. This check is required, because none of the controls in the player window can receive the focus. 
	 * @param e The associated KeyEvent object.
	 */
	protected void thisKeyTyped(KeyEvent e) {
		if (e.isAltDown()){
			if (checkOpenMenuKey(e, this.mnFile)){				
			} else if (checkOpenMenuKey(e, this.mnPreferences)){				
			} else if (checkOpenMenuKey(e, this.mnHelp)){
			}
		}
	}
	
	/**
	 * Check if the mnemonic key for the menu has been opened. If this is the case, the menu will be opened.
	 * @param e The associated KeyEvent object.
	 * @param menu The menu to check.
	 * @return True if the key has been pressed, otherwise false.
	 */
	private boolean checkOpenMenuKey(KeyEvent e, JMenu menu){
		boolean retVal = false;
		Character eChar;
		Character mChar;
		if (!menu.isShowing()){
			eChar = e.getKeyChar();
			mChar = (char)menu.getMnemonic();
			if ((int)eChar == (int)mChar){
				retVal = true;
				menu.doClick();
			}
		}
		return retVal;
	}
	/**
	 * Creates a new play list.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmPlayListNewActionPerformed(ActionEvent e) {
		playList.newPlayList();
	}
	/**
	 * Saves the play list.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmPlayListSaveActionPerformed(ActionEvent e) {
		playList.savePlayList(false);
	}
	/**
	 * Saves the play list under a new name.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmPlayListSaveAsActionPerformed(ActionEvent e) {
		playList.savePlayList(true);
	}
	/**
	 * Loads the play list.
	 * @param e The associated ActionEvent object.
	 */
	protected void mntmPlayListOpenActionPerformed(ActionEvent e) {
		playList.loadPlayList();
	}
}
