package editor;

import configuration.*;
import fontRelated.FontStyleListModel;

/**
 * The class Formatter formats setting values to strings. These strings are shown to the user in the table. 
 * @author Martin Henke
 *
 */
class Formatter {
	/**
	 * Formats a Point value.
	 * @param value The value to format.
	 * @return Returns the formatted string.
	 */
	public static String format(Point value){
		return String.format("x = %1$s, y = %2$s", value.x, value.y);
	}
	
	/**
	 * Formats a Dimension value.
	 * @param value The value to format.
	 * @return Returns the formatted string.
	 */
	public static String format(Dimension value){
		return String.format("width = %1$s, height = %2$s", value.width, value.height);		
	}
	
	/**
	 * Formats a Color value.
	 * @param value The value to format.
	 * @return Returns the formatted string.
	 */
	public static String format(Color value){
		return String.format("a = %1$s, r = %2$s, g = %3$s, b = %4$s", value.getAlpha(), value.getRed(), value.getGreen(), value.getBlue());
	}
	
	/**
	 * Formats a Point value.
	 * @param value The value to format.
	 * @return Returns the formatted string.
	 */
	public static String format(Image value){
		return ((value != null && value.getImage() != null) ? value.getFileName() : "");
	}
	
	/**
	 * Formats a Font value.
	 * @param value The value to format.
	 * @return Returns the formatted string.
	 */
	public static String format(Font value){
		return (value != null ? String.format("font = %1$s, size = %2$s, style = %3$s", value.getFamily(), value.getSize(), FontStyleListModel.getFormatString(value)) : "");
	}
	
	/**
	 * Formats a boolean value.
	 * @param value The value to format.
	 * @return Returns the formatted string.
	 */
	public static String format(boolean value){
		if (value){
			return "True";
		} else {
			return "False";
		}
	}
}
