package editor;

import generic.BackgroundPanel;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.*;

import configuration.Image;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

/**
 * The class ImageView displays the specified image unmodified, stretched or tiled.
 * @author Martin Henke
 *
 */
public class ImageView extends JDialog implements ActionListener {
	private static final long serialVersionUID = -406456390760759718L;
	
	private final BackgroundPanel contentPanel = new BackgroundPanel();
	private JButton btnClose;
	private final String imagePath;	 
	private final String imageSize;

	public ImageView(Image image) {
		setBounds(0, 0, 400, 200);
		setModal(true);
		getContentPane().setLayout(new BorderLayout());
		imagePath = image.getFileName();
		imageSize = getWidthHeightString(image.getIconWidth(), image.getIconHeight());
		setTitleText();
		contentPanel.setTransparentAdd(false);
		contentPanel.setLayout(new FlowLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPanel.setStyle(image.getStyle());
		contentPanel.setImage(image.getImage());
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				btnClose = new JButton("Close");
				btnClose.addActionListener(this);
				buttonPane.add(btnClose);
				getRootPane().setDefaultButton(btnClose);
			}
		}
		this.getRootPane().addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                // This is only called when the user releases the mouse button.
                setTitleText();
            }
        });
	}
	
	/**
	 * Gets Size string for the image (e.g. 500x300).
	 * @param width The width of the image (e.g. 500).
	 * @param height The height of the image (e.g. 300);
	 * @return Returns the Size string for the image (e.g. 500x300).
	 */
	private String getWidthHeightString(int width, int height){
		return String.valueOf(width) + "x" + String.valueOf(height);
	}
	
	/**
	 * Writes path, image size and window size into the title bar.
	 */
	private void setTitleText(){
		this.setTitle(String.format("%1$s - %2$s - %3$s", imagePath, imageSize, getWidthHeightString(this.getWidth(), this.getHeight())));
	}
	
	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnClose) {
			btnCloseActionPerformed(e);
		}
	}
	
	/**
	 * Closes the dialog.
	 * @param e The associated ActionEvent object.
	 */
	protected void btnCloseActionPerformed(ActionEvent e) {
		this.setVisible(false);
	}
}
