package editor;

import generic.SkinFileListModel;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import java.awt.Dimension;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import configuration.Skin;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.Toolkit;

/**
 * The class SkinSaveDialog allows the user to select or define a name under which to save a skin.
 * @author Martin Henke
 *
 */
public class SkinSaveDialog extends JDialog implements ListSelectionListener, DocumentListener, ActionListener, KeyListener {

	private static final long serialVersionUID = 3674860517295221853L;
	
	private static int result = JFileChooser.CANCEL_OPTION;
	
	private final JPanel contentPanel = new JPanel();
	private JTextField txtSkin;
	private JList<String> lstSkins;
	private JButton btnCancel;
	private JButton btnOK;
	private JPanel buttonPane;
	private JPanel panel;
	private JScrollPane scrollPane;

	public SkinSaveDialog() {
		setTitle("Select a Skin or enter a Name");
		setIconImage(Toolkit.getDefaultToolkit().getImage(SkinSaveDialog.class.getResource("/editor/resources/Picture File.png")));
		getContentPane().addKeyListener(this);
		addKeyListener(this);
		setBounds(100, 100, 450, 300);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.addKeyListener(this);
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(new BorderLayout(0, 0));
		panel = new JPanel();
		panel.addKeyListener(this);
		contentPanel.add(panel, BorderLayout.CENTER);
		panel.setLayout(new BorderLayout(0, 0));

		scrollPane = new JScrollPane();
		scrollPane.addKeyListener(this);
		panel.add(scrollPane, BorderLayout.CENTER);

		lstSkins = new JList<String>();
		lstSkins.addKeyListener(this);
		lstSkins.addListSelectionListener(this);
		lstSkins.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		setSkinFileListModel(false);				
		scrollPane.setViewportView(lstSkins);

		txtSkin = new JTextField();
		txtSkin.addKeyListener(this);
		txtSkin.getDocument().addDocumentListener(this);
		contentPanel.add(txtSkin, BorderLayout.SOUTH);
		txtSkin.setColumns(10);

		buttonPane = new JPanel();
		buttonPane.addKeyListener(this);
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		getContentPane().add(buttonPane, BorderLayout.SOUTH);

		btnOK = new JButton("Save");
		btnOK.addKeyListener(this);
		btnOK.addActionListener(this);
		btnOK.setMinimumSize(new Dimension(67, 23));
		btnOK.setMaximumSize(new Dimension(67, 23));
		btnOK.setPreferredSize(new Dimension(67, 23));
		btnOK.setActionCommand("OK");
		buttonPane.add(btnOK);
		getRootPane().setDefaultButton(btnOK);
		btnCancel = new JButton("Cancel");
		btnCancel.addKeyListener(this);
		btnCancel.addActionListener(this);
		btnCancel.setActionCommand("Cancel");
		buttonPane.add(btnCancel);
	}

	/**
	 * Creates a SkinFileListModel and updates the list if required.
	 * @param update Defines if the list should be updated.
	 */
	private void setSkinFileListModel(boolean update){
		SkinFileListModel model = new SkinFileListModel();
		lstSkins.setModel(model);
		if (update){
			model.update(this.lstSkins, 0, model.getSize());
		}
	}

	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnOK) {
			btnOKActionPerformed(e);
		} else if (e.getSource() == btnCancel) {
			btnCancelActionPerformed(e);
		}
	}
	/**
	 * Handles the click on btnOK. This sets the result to APPROVE_OPTION and hides the dialog.
	 * @param e The associated ActionEvent object.
	 */
	protected void btnOKActionPerformed(ActionEvent e) {
		boolean ok = false;
		String fileName = this.txtSkin.getText();
		if (!fileName.toLowerCase().endsWith(Skin.EXTENSION)){
			fileName = fileName + Skin.EXTENSION;
		}
		
		if (Skin.skinExists(fileName)){
			ok = (JOptionPane.showConfirmDialog(this, "The selected skin already exists. Do you want to overwrite it?", "Skin already exists", JOptionPane.YES_NO_OPTION) == JOptionPane.OK_OPTION);
		} else {
			ok = true;
		}
		
		if (ok){
			result = JFileChooser.APPROVE_OPTION;
			this.setVisible(false);
		}
	}
	/**
	 * Handles the click on btnCancel. This sets the result to CANCEL_OPTION and hides the dialog.
	 * @param e The associated ActionEvent object.
	 */
	protected void btnCancelActionPerformed(ActionEvent e) {
		result = JFileChooser.CANCEL_OPTION;
		this.setVisible(false);
	}

	/**
	 * Returns the name of the selected file or null.
	 * @return The name of the selected file or null.
	 */
	public String getSelectedFile(){
		if (this.txtSkin.getText().length() > 0){
			return this.txtSkin.getText();
		} else {
			return null;
		}
	}
	
	/**
	 * Handles value changes in the list.
	 * @param e The associated ListSelectionEvent object.
	 */
	public void valueChanged(ListSelectionEvent e) {
		if (e.getSource() == lstSkins) {
			listValueChanged(e);
		}
	}
	/**
	 * Handles value changes in the list lstSkins. This enters the skin name into the JTextBox txtSkin.
	 * @param e The associated ListSelectionEvent object.
	 */
	protected void listValueChanged(ListSelectionEvent e) {
		if (this.lstSkins.getSelectedIndex() > -1){
			this.txtSkin.setText(this.lstSkins.getSelectedValue());
		}
	}
	
	/**
	 * Handles text changes in the JTextBox txtSkin. This enables the button btnOK if an entry is selected, otherwise disables it.
	 * @param e The associated DocumentEvent object.
	 */
	private void textChanged(DocumentEvent e){
		this.btnOK.setEnabled(this.txtSkin.getText().length() > 0);
	}
	
	/**
	 * Shows the dialog and returns the result of selection (APPROVE_OPTION or CANCEL_OPTION).
	 * @return The result of selection (APPROVE_OPTION or CANCEL_OPTION)
	 */
	public int showDialog(){
		this.setModal(true);
		this.setVisible(true);
		return result;
	}
	
	/**
	 * Currently not used.
	 */
	@Override
	public void insertUpdate(DocumentEvent e) { }

	/**
	 * Currently not used.
	 */
	@Override
	public void removeUpdate(DocumentEvent e) { }

	/**
	 * Handles text changes in JTextBoxes. 
	 * @param e The associated DocumentEvent.
	 */
	@Override
	public void changedUpdate(DocumentEvent e) {
		textChanged(e);		
	}
	/**
	 * Currently not used.
	 */
	public void keyPressed(KeyEvent e) { }
	/**
	 * Currently not used.
	 */
	public void keyReleased(KeyEvent e) { }
	
	/**
	 * Handles key presses of the user. Updates the list if the user presses the F5 key.
	 * @param e The associated KeyEvent object.
	 */
	public void keyTyped(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_F5){
			setSkinFileListModel(true);
		}
	}
}
