package fontRelated;

import java.awt.*;

import javax.swing.*;

/**
 * The class FontFamilyListModel inherits from AbstractListModel<String> and contains all Font families found on the computer. This list static to improve performance.
 * @author Martin Henke
 *
 */
public class FontFamilyListModel extends AbstractListModel<String> {
	
	private static final long serialVersionUID = -2912415949657105254L;
	
	private static final String[] fontFamilies;
	
	/**
	 * This static constructor only fires once and loads the font families into the array fontFamilies.
	 */
	static {
		GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		fontFamilies = env.getAvailableFontFamilyNames();
	}
	
	/**
	 * Gets the number of font families.
	 */
	@Override
	public int getSize() {
		return fontFamilies.length;
	}

	/**
	 * Gets the name of the font family at the specified index.
	 */
	@Override
	public String getElementAt(int index) {
		return fontFamilies[index];
	}

}
