package fontRelated;

import configuration.Font;

import javax.swing.*;

/**
 * The class FontStyleListModel inherits from AbstractListModel<String> and contains a list of possible font styles.
 * @author Martin Henke
 *
 */
public class FontStyleListModel extends AbstractListModel<String> {
	private static final long serialVersionUID = 3937481447503987332L;

	private static final String PLAIN = "Plain";
	private static final String BOLD = "Bold";
	private static final String ITALIC = "Italic";
	private static final String BOLD_ITALIC = "Bold Italic";
	
	private static final String[] styles = {PLAIN, BOLD, ITALIC, BOLD_ITALIC};	

	/**
	 * Gets the number of styles.
	 */
	@Override
	public int getSize() {
		return styles.length;
	}

	/**
	 * Gets the name of the style at the specified index.
	 */
	@Override
	public String getElementAt(int index) {
		return styles[index];
	}
	
	/**
	 * Gets the format string for the specified Font object.
	 * @param font The Font object to get the format string for.
	 * @return The format string.
	 */
	public static String getFormatString(Font font){
		return getFormatString(font.toFont());
	}
	
	/**
	 * Gets the format string for the specified java.awt.Font object.
	 * @param font The java.awt.Font object to get the format string for.
	 * @return The format string.
	 */
	public static String getFormatString(java.awt.Font font){
		return getFormatString(font.isPlain(), font.isBold(), font.isItalic());		
	}
	
	/**
	 * Gets the format string.
	 * @param isPlain Defines if the font is plain. If this parameter is true, isBold and isItalic are false.
	 * @param isBold Defines if the font is bold.
	 * @param isItalic Defines if the font is italic.
	 * @return The format string.
	 */
	public static String getFormatString(boolean isPlain, boolean isBold, boolean isItalic){
		String retVal = "";
		if (isPlain) {
			retVal = PLAIN;
		}
		else if (isBold) {
			if (isItalic) {
				retVal = BOLD_ITALIC;
			} else {
				retVal = BOLD;
			}
		} else if (isItalic){
			retVal = ITALIC;
		}
		return retVal;
	}
	
	/**
	 * Returns the style value from a string.
	 * @param style The string specifying the style
	 * @return The style value.
	 * @throws IllegalArgumentException
	 */
	public static int getStyleFromString(String style) throws IllegalArgumentException{
		switch (style){
		case PLAIN:
			return Font.PLAIN;
		case BOLD:
			return Font.BOLD;
		case ITALIC:
			return Font.ITALIC;
		case BOLD_ITALIC:
			return Font.BOLD + Font.ITALIC;
		default:
			throw new IllegalArgumentException();
		}
	}	
}
