package fontRelated;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

/**
 * The class JFontChooser allows to choose a font in a dialog similar to the design in eclipse.
 * @author Martin Henke
 *
 */
public class JFontChooser extends JDialog implements ActionListener, ListSelectionListener {

	private static final long serialVersionUID = -6451163862440286546L;
	
	private int result = JFileChooser.CANCEL_OPTION;
	
	private final JPanel contentPanel = new JPanel();
	private JTextField txtFamily;
	private JTextField txtStyle;
	private JTextField txtSize;
	private JList<String> lstFamily;
	private JList<String> lstStyle;
	private JList<Integer> lstSize;
	private JLabel lblFamily;
	private JLabel lblStyle;
	private JLabel lblSize;
	private JButton btnOK;
	private JButton btnCancel;
	private JScrollPane scpFamily;
	private JScrollPane scpStyle;
	private JScrollPane scpSize;
	
	/**
	 * Initializes the dialog.
	 */
	private JFontChooser() {
		setMinimumSize(new Dimension(225, 225));
		setBounds(100, 100, 450, 274);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		GridBagLayout gbl_contentPanel = new GridBagLayout();
		gbl_contentPanel.columnWidths = new int[] {180, 63, 50};
		gbl_contentPanel.rowHeights = new int[]{0, 0, 0, 0, 0, 0};
		gbl_contentPanel.columnWeights = new double[]{1.0, 0.0, 0.0};
		gbl_contentPanel.rowWeights = new double[]{0.0, 0.0, 0.0, 1.0, 1.0, Double.MIN_VALUE};
		contentPanel.setLayout(gbl_contentPanel);
		lblFamily = new JLabel("Family");
		lblFamily.setHorizontalAlignment(SwingConstants.LEFT);
		GridBagConstraints gbc_lblFamily = new GridBagConstraints();
		gbc_lblFamily.fill = GridBagConstraints.BOTH;
		gbc_lblFamily.anchor = GridBagConstraints.WEST;
		gbc_lblFamily.insets = new Insets(0, 0, 5, 5);
		gbc_lblFamily.gridx = 0;
		gbc_lblFamily.gridy = 0;
		contentPanel.add(lblFamily, gbc_lblFamily);

		lblStyle = new JLabel("Style");
		GridBagConstraints gbc_lblStyle = new GridBagConstraints();
		gbc_lblStyle.anchor = GridBagConstraints.WEST;
		gbc_lblStyle.insets = new Insets(0, 0, 5, 5);
		gbc_lblStyle.gridx = 1;
		gbc_lblStyle.gridy = 0;
		contentPanel.add(lblStyle, gbc_lblStyle);

		lblSize = new JLabel("Size");
		lblSize.setMinimumSize(new Dimension(50, 14));
		lblSize.setMaximumSize(new Dimension(50, 14));
		lblSize.setSize(new Dimension(50, 0));
		lblSize.setPreferredSize(new Dimension(50, 14));
		GridBagConstraints gbc_lblSize = new GridBagConstraints();
		gbc_lblSize.fill = GridBagConstraints.BOTH;
		gbc_lblSize.anchor = GridBagConstraints.WEST;
		gbc_lblSize.insets = new Insets(0, 0, 5, 0);
		gbc_lblSize.gridx = 2;
		gbc_lblSize.gridy = 0;
		contentPanel.add(lblSize, gbc_lblSize);

		txtFamily = new JTextField();
		txtFamily.setEditable(false);
		GridBagConstraints gbc_txtFamily = new GridBagConstraints();
		gbc_txtFamily.insets = new Insets(0, 0, 5, 5);
		gbc_txtFamily.anchor = GridBagConstraints.WEST;
		gbc_txtFamily.fill = GridBagConstraints.BOTH;
		gbc_txtFamily.gridx = 0;
		gbc_txtFamily.gridy = 1;
		contentPanel.add(txtFamily, gbc_txtFamily);
		txtFamily.setColumns(10);

		txtStyle = new JTextField();
		txtStyle.setSize(new Dimension(63, 20));
		txtStyle.setPreferredSize(new Dimension(63, 20));
		txtStyle.setMinimumSize(new Dimension(63, 20));
		txtStyle.setMaximumSize(new Dimension(63, 2147483647));
		txtStyle.setEditable(false);
		GridBagConstraints gbc_txtStyle = new GridBagConstraints();
		gbc_txtStyle.fill = GridBagConstraints.BOTH;
		gbc_txtStyle.insets = new Insets(0, 0, 5, 5);
		gbc_txtStyle.anchor = GridBagConstraints.WEST;
		gbc_txtStyle.gridx = 1;
		gbc_txtStyle.gridy = 1;
		contentPanel.add(txtStyle, gbc_txtStyle);
		txtStyle.setColumns(11);

		txtSize = new JTextField();
		txtSize.setSize(new Dimension(50, 20));
		txtSize.setMaximumSize(new Dimension(50, 2147483647));
		txtSize.setPreferredSize(new Dimension(50, 20));
		txtSize.setMinimumSize(new Dimension(50, 20));
		txtSize.setEditable(false);
		GridBagConstraints gbc_txtSize = new GridBagConstraints();
		gbc_txtSize.fill = GridBagConstraints.BOTH;
		gbc_txtSize.insets = new Insets(0, 0, 5, 0);
		gbc_txtSize.anchor = GridBagConstraints.NORTH;
		gbc_txtSize.gridx = 2;
		gbc_txtSize.gridy = 1;
		contentPanel.add(txtSize, gbc_txtSize);
		txtSize.setColumns(3);

		scpSize = new JScrollPane();
		scpSize.setMinimumSize(new Dimension(50, 4));
		scpSize.setMaximumSize(new Dimension(50, 32767));
		scpSize.setSize(new Dimension(50, 0));
		scpSize.setPreferredSize(new Dimension(50, 4));
		scpSize.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		GridBagConstraints gbc_scpSize = new GridBagConstraints();
		gbc_scpSize.gridheight = 3;
		gbc_scpSize.fill = GridBagConstraints.BOTH;
		gbc_scpSize.gridx = 2;
		gbc_scpSize.gridy = 2;
		contentPanel.add(scpSize, gbc_scpSize);

		lstSize = new JList<Integer>();
		lstSize.addListSelectionListener(this);
		lstSize.setModel(new FontSizeListModel());
		lstSize.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scpSize.setViewportView(lstSize);

		scpFamily = new JScrollPane();
		scpFamily.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		GridBagConstraints gbc_scpFamily = new GridBagConstraints();
		gbc_scpFamily.fill = GridBagConstraints.BOTH;
		gbc_scpFamily.gridheight = 3;
		gbc_scpFamily.insets = new Insets(0, 0, 0, 5);
		gbc_scpFamily.gridx = 0;
		gbc_scpFamily.gridy = 2;

		contentPanel.add(scpFamily, gbc_scpFamily);

		lstFamily = new JList<String>();
		lstFamily.addListSelectionListener(this);
		lstFamily.setModel(new FontFamilyListModel());
		lstFamily.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scpFamily.setViewportView(lstFamily);

		scpStyle = new JScrollPane();
		scpStyle.setPreferredSize(new Dimension(63, 4));
		scpStyle.setMaximumSize(new Dimension(63, 32767));
		scpStyle.setMinimumSize(new Dimension(63, 24));
		scpStyle.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		GridBagConstraints gbc_scpStyle = new GridBagConstraints();
		gbc_scpStyle.insets = new Insets(0, 0, 0, 5);
		gbc_scpStyle.fill = GridBagConstraints.BOTH;
		gbc_scpStyle.gridheight = 3;
		gbc_scpStyle.gridx = 1;
		gbc_scpStyle.gridy = 2;
		contentPanel.add(scpStyle, gbc_scpStyle);

		lstStyle = new JList<String>();
		lstStyle.setSize(new Dimension(63, 0));
		lstStyle.setPreferredSize(new Dimension(63, 0));
		lstStyle.setMinimumSize(new Dimension(63, 0));
		lstStyle.setMaximumSize(new Dimension(63, 0));
		lstStyle.setBounds(new Rectangle(0, 0, 63, 0));
		lstStyle.addListSelectionListener(this);
		lstStyle.setModel(new FontStyleListModel());
		lstStyle.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scpStyle.setViewportView(lstStyle);

		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		getContentPane().add(buttonPane, BorderLayout.SOUTH);

		btnOK = new JButton("OK");
		btnOK.addActionListener(this);
		btnOK.setEnabled(false);
		btnOK.setActionCommand("OK");
		btnOK.setEnabled(false);
		buttonPane.add(btnOK);
		getRootPane().setDefaultButton(btnOK);

		btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(this);
		btnCancel.setActionCommand("Cancel");
		buttonPane.add(btnCancel);
	}
	
	/**
	 * Initializes the dialog and sets title and initial font.
	 * @param title The title for the dialog.
	 * @param initialFont The font to choose when the dialog is opened.
	 */
	public JFontChooser(String title, Font initialFont){
		this();
		this.setTitle(title);
		setInitialFont(initialFont);
	}
	
	/**
	 * Sets the font family, font size and font style.
	 * @param initialFont The font to find in the lists.
	 */
	private void setInitialFont(Font initialFont){
		this.lstFamily.setSelectedValue(initialFont.getFamily(), true);
		this.lstSize.setSelectedValue(initialFont.getSize(), true);
		this.lstStyle.setSelectedValue(FontStyleListModel.getFormatString(initialFont), true);
	}
	
	/**
	 * Shows the dialog and returns the status (APPROVED_OPTION or CANCEL_OPTION).
	 * @return The status (APPROVED_OPTION or CANCEL_OPTION).
	 */
	public int showDialog(){
		this.setModal(true);
		this.setVisible(true);
		return result;
	}
	
	/**
	 * Gets a Font object of the selected entries.
	 */
	public Font getFont(){
		return new Font(this.txtFamily.getText(), FontStyleListModel.getStyleFromString(this.txtStyle.getText()), Integer.parseInt(this.txtSize.getText()));
	}
	
	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnCancel) {
			btnCancelActionPerformed(e);
		} else if (e.getSource() == btnOK) {
			btnOKActionPerformed(e);
		}
	}
	/**
	 * Sets the result to APPROVE_OPTION and hides the dialog. 
	 * @param e The associated ActionEvent object.
	 */
	protected void btnOKActionPerformed(ActionEvent e) {
		this.result = JFileChooser.APPROVE_OPTION;
		this.setVisible(false);
	}
	
	/**
	 *  Sets the result to CANCEL_OPTION and hides the dialog.
	 * @param e The associated ActionEvent object.
	 */
	protected void btnCancelActionPerformed(ActionEvent e) {
		this.result = JFileChooser.CANCEL_OPTION;
		this.setVisible(false);
	}
	
	/**
	 * Handles list selection changes
	 * @param e The associated ListSelectionEvent object.
	 */
	public void valueChanged(ListSelectionEvent e) {
		if (e.getSource() == lstSize) {
			lstSizeValueChanged(e);
		} else if (e.getSource() == lstFamily) {
			lstFamilyValueChanged(e);
		} else if (e.getSource() == lstStyle) {
			lstStyleValueChanged(e);
		}
	}
	
	/**
	 * Sets the selected style into the JTextBox txtStyle and enables/disables the OK button. 
	 * @param e The associated ListSelectionEvent object.
	 */
	protected void lstStyleValueChanged(ListSelectionEvent e) {
		this.txtStyle.setText(this.lstStyle.getSelectedValue());
		enableOKButton();
	}
	/**
	 * Sets the selected font family into the JTextBox txtFamily and enables/disables the OK button. 
	 * @param e The associated ListSelectionEvent object.
	 */
	protected void lstFamilyValueChanged(ListSelectionEvent e) {
		this.txtFamily.setText(this.lstFamily.getSelectedValue());
		enableOKButton();
	}
	/**
	 * Sets the selected size into the JTextBox txtSize and enables/disables the OK button. 
	 * @param e The associated ListSelectionEvent object.
	 */
	protected void lstSizeValueChanged(ListSelectionEvent e) {
		this.txtSize.setText(String.valueOf(this.lstSize.getSelectedValue()));
		enableOKButton();
	}
	
	/**
	 * Enables the OK button, if in each of the lists lstFamily, lstSize and lstSize an entry is selected, otherwise diables it. 
	 */
	private void enableOKButton(){
		btnOK.setEnabled(this.lstFamily.getSelectedIndex() > -1 && this.lstSize.getSelectedIndex() > -1 && this.lstStyle.getSelectedIndex() > -1);
	}
}
