package panels;

import javax.swing.*;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * The class BoolPanel inherits from JPanel and provides the functionality to choose a boolean value.
 * @author Martin Henke
 *
 */
public class BoolPanel extends JPanel implements ActionListener {
	
	private static final long serialVersionUID = -3756590181990821182L;

	private JRadioButton rdbtnYes;
	private JRadioButton rdbtnNo;

	/**
	 * Initializes the panel.
	 */
	public BoolPanel() {
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 0};
		gridBagLayout.rowHeights = new int[]{0, 0, 0};
		gridBagLayout.columnWeights = new double[]{0.0, Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 0.0, Double.MIN_VALUE};
		setLayout(gridBagLayout);
		
		rdbtnYes = new JRadioButton("True");
		rdbtnYes.addActionListener(this);
		GridBagConstraints gbc_rdbtnYes = new GridBagConstraints();
		gbc_rdbtnYes.insets = new Insets(0, 0, 5, 0);
		gbc_rdbtnYes.gridx = 0;
		gbc_rdbtnYes.gridy = 0;
		add(rdbtnYes, gbc_rdbtnYes);
		
		rdbtnNo = new JRadioButton("False");
		rdbtnNo.addActionListener(this);
		GridBagConstraints gbc_rdbtnNo = new GridBagConstraints();
		gbc_rdbtnNo.gridx = 0;
		gbc_rdbtnNo.gridy = 1;
		add(rdbtnNo, gbc_rdbtnNo);
	}

	/**
	 * Gets the selected value.
	 * @return The selected value.
	 */
	public boolean getValue(){
		return rdbtnYes.isSelected();
	}
	
	/**
	 * Sets the value to the JRadioButtons.
	 * @param value The boolean value to set to the JRadiobuttons.
	 */
	public void setValue(boolean value){
		rdbtnYes.setSelected(value);
		rdbtnNo.setSelected(!value);
	}

	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == rdbtnNo) {
			rdbtnNoActionPerformed(e);
		} else if (e.getSource() == rdbtnYes) {
			rdbtnYesActionPerformed(e);
		}
	}
	/**
	 * Handles the click on the "Yes" JRadioButton and sets the "No" JRadioButton opposite.
	 * @param e The associated ActionEvent object.
	 */
	protected void rdbtnYesActionPerformed(ActionEvent e) {
		rdbtnNo.setSelected(!rdbtnYes.isSelected());
	}
	/**
	 * Handles the click on the "No" JRadioButton and sets the "Yes" JRadioButton opposite.
	 * @param e The associated ActionEvent object.
	 */
	protected void rdbtnNoActionPerformed(ActionEvent e) {
		rdbtnYes.setSelected(!rdbtnNo.isSelected());
	}	
}
