package panels;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import configuration.Color;

/**
 * The class ColorPanel inherits from JPanel and provides the functionality to choose a color.
 * @author Martin Henke
 *
 */
public class ColorPanel extends JPanel implements ChangeListener, ActionListener {
	private static final long serialVersionUID = -3804914953517192663L;
	
	private JLabel lblRed;
	private JSpinner red;
	private JLabel lblGreen;
	private JSpinner green;
	private JLabel lblBlue;
	private JSpinner blue;
	private JButton btnChoose;
	private JLabel lblColor;
	
	/**
	 * Initializes the panel.
	 */
	public ColorPanel() {
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 0, 0, 0, 0, 0, 0};
		gridBagLayout.rowHeights = new int[]{0, 0, 0, 0};
		gridBagLayout.columnWeights = new double[]{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 0.0, 0.0, Double.MIN_VALUE};
		setLayout(gridBagLayout);
		
		lblRed = new JLabel("Red");
		lblRed.setFont(new Font("Tahoma", Font.PLAIN, 11));
		GridBagConstraints gbc_lblRed = new GridBagConstraints();
		gbc_lblRed.insets = new Insets(0, 0, 5, 5);
		gbc_lblRed.gridx = 0;
		gbc_lblRed.gridy = 0;
		add(lblRed, gbc_lblRed);
		
		red = new JSpinner();
		red.setFont(new Font("Tahoma", Font.PLAIN, 11));
		red.addChangeListener(this);
		red.setModel(new SpinnerNumberModel(0, null, 255, 1));
		GridBagConstraints gbc_red = new GridBagConstraints();
		gbc_red.insets = new Insets(0, 0, 5, 5);
		gbc_red.gridx = 1;
		gbc_red.gridy = 0;
		add(red, gbc_red);
		
		lblColor = new JLabel("");
		lblColor.setFocusable(false);
		lblColor.setOpaque(true);
		lblColor.setBorder(new BevelBorder(BevelBorder.LOWERED, null, null, null, null));
		GridBagConstraints gbc_lbColor = new GridBagConstraints();
		gbc_lbColor.fill = GridBagConstraints.BOTH;
		gbc_lbColor.weightx = 1.0;
		gbc_lbColor.gridwidth = 3;
		gbc_lbColor.gridheight = 3;
		gbc_lbColor.insets = new Insets(0, 0, 5, 0);
		gbc_lbColor.gridx = 5;
		gbc_lbColor.gridy = 0;
		add(lblColor, gbc_lbColor);
		
		lblGreen = new JLabel("Green");
		lblGreen.setFont(new Font("Tahoma", Font.PLAIN, 11));
		GridBagConstraints gbc_lblGreen = new GridBagConstraints();
		gbc_lblGreen.insets = new Insets(0, 0, 5, 5);
		gbc_lblGreen.gridx = 0;
		gbc_lblGreen.gridy = 1;
		add(lblGreen, gbc_lblGreen);
		
		green = new JSpinner();
		green.setFont(new Font("Tahoma", Font.PLAIN, 11));
		green.addChangeListener(this);
		green.setModel(new SpinnerNumberModel(0, null, 255, 1));
		GridBagConstraints gbc_green = new GridBagConstraints();
		gbc_green.insets = new Insets(0, 0, 5, 5);
		gbc_green.gridx = 1;
		gbc_green.gridy = 1;
		add(green, gbc_green);
		
		btnChoose = new JButton("Choose");
		btnChoose.setFont(new Font("Tahoma", Font.PLAIN, 11));
		btnChoose.addActionListener(this);
		GridBagConstraints gbc_btnChoose = new GridBagConstraints();
		gbc_btnChoose.insets = new Insets(0, 0, 5, 5);
		gbc_btnChoose.gridx = 3;
		gbc_btnChoose.gridy = 1;
		add(btnChoose, gbc_btnChoose);
		
		lblBlue = new JLabel("Blue");
		lblBlue.setFont(new Font("Tahoma", Font.PLAIN, 11));
		GridBagConstraints gbc_lblBlue = new GridBagConstraints();
		gbc_lblBlue.insets = new Insets(0, 0, 0, 5);
		gbc_lblBlue.gridx = 0;
		gbc_lblBlue.gridy = 2;
		add(lblBlue, gbc_lblBlue);
		
		blue = new JSpinner();
		blue.setFont(new Font("Tahoma", Font.PLAIN, 11));
		blue.addChangeListener(this);
		blue.setModel(new SpinnerNumberModel(0, null, 255, 1));
		GridBagConstraints gbc_blue = new GridBagConstraints();
		gbc_blue.insets = new Insets(0, 0, 0, 5);
		gbc_blue.gridx = 1;
		gbc_blue.gridy = 2;
		add(blue, gbc_blue);
	}

	/**
	 * Handles changes of the JSpinners red, green and blue.
	 * @param e The associated ChangeEvent object.
	 */
	public void stateChanged(ChangeEvent e) {
		if (e.getSource() == red) {
			redStateChanged(e);
		} else if (e.getSource() == green) {
			greenStateChanged(e);
		} else if (e.getSource() == blue) {
			blueStateChanged(e);
		}
	}
	
	/**
	 * Handles changes of the JSpinner "red" and calls setValue().
	 * @param e The associated ChangeEvent object.
	 */
	protected void blueStateChanged(ChangeEvent e) {
		setValue();
	}
	/**
	 * Handles changes of the JSpinner "green" and calls setValue().
	 * @param e The associated ChangeEvent object.
	 */
	protected void greenStateChanged(ChangeEvent e) {
		setValue();
	}
	/**
	 * Handles changes of the JSpinner "blue" and calls setValue().
	 * @param e The associated ChangeEvent object.
	 */
	protected void redStateChanged(ChangeEvent e) {
		setValue();
	}
	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnChoose) {
			btnChooseActionPerformed(e);
		}
	}
	/**
	 * Shows the JColorChooser dialog when the user clicks on the "Choose" button.
	 * @param e The associated ActionEvent object.
	 */
	protected void btnChooseActionPerformed(ActionEvent e) {
		setValue(new Color(JColorChooser.showDialog(this, "Choose a Color", lblColor.getBackground())));
	}

	/**
	 * Sets the specified value to the JSpinners and calls setValue().
	 * @param color The color to set.
	 */
	public void setValue(Color color){
		red.setValue(color.getRed());
		green.setValue(color.getGreen());
		blue.setValue(color.getBlue());
		setValue();
	}

	/**
	 * Reads the values from the JSpinners and sets the background for the label lblColor.
	 */
	private void setValue(){
		int r = (int) red.getValue();
		int g = (int) green.getValue();
		int b = (int) blue.getValue();
		Color color = new Color(r, g, b);
		this.lblColor.setBackground(color.toColor());
	}
	
	/**
	 * Gets the selected color.
	 * @return The selected color.
	 */
	public Color getValue() { return new Color(this.lblColor.getBackground()); }
}
