package panels;

import java.awt.*;
import javax.swing.*;
import configuration.Dimension;

/**
 * The class DimensionPanel inherits from JPanel and provides the functionality to choose a size.
 * @author Martin Henke
 *
 */
public class DimensionPanel extends JPanel {

	private static final long serialVersionUID = 5865154655935130200L;
	private JLabel lblWidth;
	private JSpinner width;
	private JLabel lblHeight;
	private JSpinner height;

	/**
	 * Initializes the panel.
	 */
	public DimensionPanel(){
		setLayout(null);
		
		lblWidth = new JLabel("Width");
		lblWidth.setFont(new Font("Tahoma", Font.PLAIN, 11));
		lblWidth.setBounds(0, 2, 42, 14);
		add(lblWidth);
		
		width = new JSpinner();
		width.setFont(new Font("Tahoma", Font.PLAIN, 11));
		width.setBounds(52, 0, 62, 18);
		add(width);
		
		lblHeight = new JLabel("Height");
		lblHeight.setFont(new Font("Tahoma", Font.PLAIN, 11));
		lblHeight.setBounds(0, 25, 42, 14);
		add(lblHeight);
		
		height = new JSpinner();
		height.setFont(new Font("Tahoma", Font.PLAIN, 11));
		height.setBounds(52, 23, 62, 18);
		add(height);
		
		setMinimumValue(1);
	}
	
	/**
	 * Sets the minimum value for the JSpinners.
	 * @param minimumValue The minimum value for the JSpinners.
	 */
	public void setMinimumValue(int minimumValue){
		width.setModel(new SpinnerNumberModel(new Integer(1), new Integer(minimumValue), null, new Integer(1)));
		height.setModel(new SpinnerNumberModel(new Integer(1), new Integer(minimumValue), null, new Integer(1)));
	}
	
	/**
	 * Gets the selected size.
	 * @return The selected size.
	 */
	public Dimension getValue(){
		return new Dimension((int)width.getValue(),(int) height.getValue());
	}
	/**
	 * Sets the size.
	 * @param value The size to set.
	 */
	public void setValue(Dimension value){
		width.setValue(value.width);
		height.setValue(value.height);
	}
}
