package panels;

import javax.swing.JPanel;
import java.awt.GridBagLayout;
import javax.swing.JLabel;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.filechooser.FileFilter;

import generic.BackgroundPanel.Style;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;

import configuration.Image;
import editor.ImageView;

import java.awt.Font;

/**
 * The class ImagePanel inherits from JPanel and provides the functionality to select an image.
 * @author Martin Henke
 *
 */
public class ImagePanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = -6326224811330172065L;
	
	private JLabel lblImage;
	private JLabel lblNewLabel;
	private JButton btnChoose;
	private JTextField txtPath;
	private JLabel lblDisplayMode;
	private JComboBox<Style> cmbDisplayMode;
	private JButton btnClear;
	private JButton btnPreview;
	private Image image = null;
	private static JFileChooser fileChooser;
	private JLabel lblSize;
	private JLabel lblImageSize;
	
	static {
		fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new FileFilter(){

			@Override
			public boolean accept(File f) {
				return f.isDirectory() || f.getName().toLowerCase().endsWith(".png");
			}

			@Override
			public String getDescription() {
				return "Portable Network Graphics (*.png)";
			}			
		});
		fileChooser.setDialogTitle("Select a PNG picture");
		fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setMultiSelectionEnabled(false);
		fileChooser.setAcceptAllFileFilterUsed(false);
	}
	
	/**
	 * Initializes the panel.
	 */
	public ImagePanel(){
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 0, 0, 0, 0};
		gridBagLayout.rowHeights = new int[]{0, 0, 0, 0, 0};
		gridBagLayout.columnWeights = new double[]{0.0, 1.0, 0.0, 0.0, Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE};
		setLayout(gridBagLayout);
		
		lblImage = new JLabel("Image");
		lblImage.setFont(new Font("Tahoma", Font.PLAIN, 11));
		GridBagConstraints gbc_lblImage = new GridBagConstraints();
		gbc_lblImage.anchor = GridBagConstraints.WEST;
		gbc_lblImage.insets = new Insets(0, 0, 5, 5);
		gbc_lblImage.gridx = 0;
		gbc_lblImage.gridy = 0;
		add(lblImage, gbc_lblImage);
		
		txtPath = new JTextField();
		txtPath.setFont(new Font("Tahoma", Font.PLAIN, 11));
		txtPath.setEditable(false);
		GridBagConstraints gbc_txtPath = new GridBagConstraints();
		gbc_txtPath.anchor = GridBagConstraints.WEST;
		gbc_txtPath.insets = new Insets(0, 0, 5, 5);
		gbc_txtPath.fill = GridBagConstraints.HORIZONTAL;
		gbc_txtPath.gridx = 1;
		gbc_txtPath.gridy = 0;
		add(txtPath, gbc_txtPath);
		txtPath.setColumns(45);
		
		lblNewLabel = new JLabel("");
		GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
		gbc_lblNewLabel.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel.gridx = 2;
		gbc_lblNewLabel.gridy = 0;
		add(lblNewLabel, gbc_lblNewLabel);
		
		btnChoose = new JButton("Choose");
		btnChoose.setFont(new Font("Tahoma", Font.PLAIN, 11));
		btnChoose.addActionListener(this);
		GridBagConstraints gbc_btnChoose = new GridBagConstraints();
		gbc_btnChoose.fill = GridBagConstraints.BOTH;
		gbc_btnChoose.insets = new Insets(0, 0, 5, 0);
		gbc_btnChoose.gridx = 3;
		gbc_btnChoose.gridy = 0;
		add(btnChoose, gbc_btnChoose);
		
		lblDisplayMode = new JLabel("Display Mode");
		lblDisplayMode.setFont(new Font("Tahoma", Font.PLAIN, 11));
		GridBagConstraints gbc_lblDisplayMode = new GridBagConstraints();
		gbc_lblDisplayMode.anchor = GridBagConstraints.EAST;
		gbc_lblDisplayMode.insets = new Insets(0, 0, 5, 5);
		gbc_lblDisplayMode.gridx = 0;
		gbc_lblDisplayMode.gridy = 1;
		add(lblDisplayMode, gbc_lblDisplayMode);
		
		cmbDisplayMode = new JComboBox<Style>();
		cmbDisplayMode.setFont(new Font("Tahoma", Font.PLAIN, 11));
		cmbDisplayMode.setModel(new DefaultComboBoxModel<Style>(Style.values()));
		GridBagConstraints gbc_cmbDisplayMode = new GridBagConstraints();
		gbc_cmbDisplayMode.insets = new Insets(0, 0, 5, 5);
		gbc_cmbDisplayMode.fill = GridBagConstraints.HORIZONTAL;
		gbc_cmbDisplayMode.gridx = 1;
		gbc_cmbDisplayMode.gridy = 1;
		add(cmbDisplayMode, gbc_cmbDisplayMode);
		
		btnClear = new JButton("Clear");
		btnClear.setFont(new Font("Tahoma", Font.PLAIN, 11));
		btnClear.addActionListener(this);
		GridBagConstraints gbc_btnClear = new GridBagConstraints();
		gbc_btnClear.fill = GridBagConstraints.BOTH;
		gbc_btnClear.insets = new Insets(0, 0, 5, 0);
		gbc_btnClear.gridx = 3;
		gbc_btnClear.gridy = 1;
		add(btnClear, gbc_btnClear);
		
		btnPreview = new JButton("Preview...");
		btnPreview.setEnabled(false);
		btnPreview.addActionListener(this);
		btnPreview.setFont(new Font("Tahoma", Font.PLAIN, 11));
		GridBagConstraints gbc_btnPreview = new GridBagConstraints();
		gbc_btnPreview.insets = new Insets(0, 0, 5, 0);
		gbc_btnPreview.fill = GridBagConstraints.BOTH;
		gbc_btnPreview.gridx = 3;
		gbc_btnPreview.gridy = 2;
		add(btnPreview, gbc_btnPreview);
		
		lblSize = new JLabel("Size");
		GridBagConstraints gbc_lblSize = new GridBagConstraints();
		gbc_lblSize.anchor = GridBagConstraints.WEST;
		gbc_lblSize.insets = new Insets(0, 0, 0, 5);
		gbc_lblSize.gridx = 0;
		gbc_lblSize.gridy = 3;
		add(lblSize, gbc_lblSize);
		
		lblImageSize = new JLabel();
		setImageSize(0, 0);
		GridBagConstraints gbc_lblImageSize = new GridBagConstraints();
		gbc_lblImageSize.fill = GridBagConstraints.BOTH;
		gbc_lblImageSize.anchor = GridBagConstraints.WEST;
		gbc_lblImageSize.insets = new Insets(0, 0, 0, 5);
		gbc_lblImageSize.gridx = 1;
		gbc_lblImageSize.gridy = 3;
		add(lblImageSize, gbc_lblImageSize);
		
	}
	
	/**
	 * Gets the selected image.
	 * @return The selected image.
	 */
	public Image getValue(){
		if (image != null){
			image.setStyle((Style)cmbDisplayMode.getSelectedItem());
		}
		return image;
	}
	
	/**
	 * Sets the image.
	 * @param value The image to set.
	 */
	public void setValue(Image value){
		this.image = value;
		if (value != null){
			this.txtPath.setText(value.getFileName());
			this.cmbDisplayMode.setSelectedItem(value.getStyle());
			setImageSize(value.getIconWidth(), value.getIconHeight());
			this.btnPreview.setEnabled(true);			
		} else {
			clear();		
		}		
	}

	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnChoose) {
			btnChooseActionPerformed(e);
		} else if (e.getSource() == btnClear) {
			btnClearActionPerformed(e);
		} else if (e.getSource() == btnPreview) {
			btnPreviewActionPerformed(e);
		} 
	}
	/**
	 * Opens the JFileChooser dialog for PNG files and loads the image.
	 * @param e The associated ActionEvent object.
	 */
	protected void btnChooseActionPerformed(ActionEvent e) {
		if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION && fileChooser.getSelectedFile().getName().toLowerCase().endsWith(".png")){
			try
			{
				setValue(new Image(fileChooser.getSelectedFile().getAbsolutePath()));
			} catch (Exception ex){
				clear();
				JOptionPane.showMessageDialog(this, "The selected file cannot be opened or is not an image file.", "Error loading image", JOptionPane.WARNING_MESSAGE);
			}
		}		
	}
	
	/**
	 * Calls clear(). 
	 * @param e The associated ActionEvent object.
	 */
	protected void btnClearActionPerformed(ActionEvent e) {
		clear();
	}
	
	/**
	 * Clears the image and resets the UI.
	 */
	private void clear(){
		image = null;
		txtPath.setText("");
		this.btnPreview.setEnabled(false);
		setImageSize(0, 0);
	}
	
	/**
	 * Opens an ImageView dialog to display the selected image. 
	 * @param e The associated ActionEvent object.
	 */
	protected void btnPreviewActionPerformed(ActionEvent e) {
		ImageView iv = new ImageView(getValue());
		iv.setModal(true);
		iv.setVisible(true);
		if (iv != null){
			iv.dispose();
		}
	}
	
	/**
	 * Sets the specified width and height as text to the label lblImageSize.
	 * @param width The image width.
	 * @param height The image height.
	 */
	private void setImageSize(int width, int height){
		this.lblImageSize.setText(String.format("%1$s x %2$s", width, height));
	}
}
