package panels;

import java.awt.*;
import javax.swing.*;
import configuration.Point;

/**
 * The class PointPanel inherits from JPanel and provides the functionality to enter a location.
 * @author Martin Henke
 *
 */
public class PointPanel extends JPanel {
	private static final long serialVersionUID = 5523669559492651951L;
	private JLabel lblLeft;
	private JSpinner left;
	private JLabel lblTop;
	private JSpinner top;

	public PointPanel(){
		setLayout(null);
		
		lblLeft = new JLabel("Left");
		lblLeft.setFont(new Font("Tahoma", Font.PLAIN, 11));
		lblLeft.setBounds(0, 2, 35, 14);
		add(lblLeft);
		
		left = new JSpinner();
		left.setFont(new Font("Tahoma", Font.PLAIN, 11));
		left.setBounds(45, 0, 62, 18);
		add(left);
		
		lblTop = new JLabel("Top");
		lblTop.setFont(new Font("Tahoma", Font.PLAIN, 11));
		lblTop.setBounds(0, 25, 35, 14);
		add(lblTop);
		
		top = new JSpinner();
		top.setFont(new Font("Tahoma", Font.PLAIN, 11));
		top.setBounds(45, 23, 62, 18);
		add(top);
		
		setMinimumValue(0);
	}
	
	/**
	 * Sets the minimum value for the JSpinners.
	 * @param minimumValue The minimum value for the JSpinners.
	 */
	public void setMinimumValue(int minimumValue){
		left.setModel(new SpinnerNumberModel(new Integer(1), new Integer(minimumValue), null, new Integer(1)));
		top.setModel(new SpinnerNumberModel(new Integer(1), new Integer(minimumValue), null, new Integer(1)));
	}
	
	/**
	 * Gets the selected location.
	 * @return The selected location.
	 */
	public Point getValue(){
		return new Point((int)left.getValue(),(int) top.getValue());
	}
	/**
	 * Sets the location.
	 * @param value The location to set.
	 */
	public void setValue(Point value){
		left.setValue(value.x);
		top.setValue(value.y);
	}
}
