package configuration;

import java.io.*;
import javax.xml.bind.annotation.XmlTransient;

/**
 * The class Button inherits from ImageControl and allows XML serialization of the settings for the buttons displayed by MP3Player.
 * @author Martin Henke
 *
 */
public class Button extends ImageControl implements Serializable {
	
	private static final long serialVersionUID = -9087815281264648030L;
	
	private transient Image mask;
	
	/**
	 * The empty constructor is required for XML serialization and should not be used in your code.
	 */
	public Button(){ 
		super();
	}
	
	/**
	 * The constructor allows to specify this name as a parameter.
	 * @param name The name of this object.
	 */
	public Button(String name){
		super(name);
	}
	
	/**
	 * Gets the mask image. The mask image defines the clickable area of the button. The mask image is not serialized.
	 * @return Returns the mask image.
	 */
	public Image getMask() { return mask; }
	/**
	 * Sets the mask image. The mask image defines the clickable area of the button. The mask image is not serialized.
	 * @param mask The mask image.
	 */
	@XmlTransient
	public void setMask(Image mask) { this.mask = mask; }
	
	/**
	 * Loads the mask image from the specified directory.
	 * @param directory The directory the mask image is located in.
	 */
	public void loadMask(String directory){
		// Get the image file of the specified directory and add "Mask" to the file name.
		File f = getImageFile(directory, "Mask");
		
		// Load the mask image
		if (f.exists() && !f.isDirectory()){
			mask = new Image(f.getAbsolutePath());
		} else {
			mask = null;
		}
	}
	
	/**
	 * Save the mask image into the specified directory
	 * @param directory The directory to save the mask file to.
	 * @throws IOException
	 */
	public void saveMask(String directory) throws IOException {
		// Only save image if a mask is set.
		if (mask != null){
			File f = getImageFile(directory, "Mask");
			copyImage(mask, f);
		}
	}
}
