package configuration;

import java.io.Serializable;

import javax.xml.bind.annotation.XmlAttribute;

/**
 * This class Dimension allows XML serialization. Creation of this class was necessary as the java.awt.Dimension class creates a Stack Overflow Exception.
 * @author Martin Henke
 *
 */
public class Dimension implements Serializable {
	private static final long serialVersionUID = -1492757445388803255L;

	/**
	 * The width of the dimension.
	 */
	@XmlAttribute(name="width", required=true)
	public int width;
	/**
	 * The height of the dimension.
	 */
	@XmlAttribute(name="height", required=true)
	public int height;
	
	/**
	 * Initializes the class with width=0 and height=0.
	 */
	public Dimension() {
		this(0, 0);
	}
	
	/**
	 * Initializes the class with the specified width and height.
	 * @param width The width to use.
	 * @param height The height to use.
	 */
	public Dimension(int width, int height){
		this.width = width;
		this.height = height;
	}
	
	/**
	 * Initializes the class with the specified java.awt.Dimension object.
	 * @param dimension The Dimension object to provide width and height.
	 */
	public Dimension(java.awt.Dimension dimension){
		this.width = dimension.width;
		this.height = dimension.height;
	}
	
	/**
	 * Converts this Dimension object to a java.awt.Dimension object.
	 * @return A java.awt.Dimension object.
	 */
	public java.awt.Dimension toDimension(){
		return new java.awt.Dimension(width, height);
	}

}
