package configuration;

import generic.BackgroundPanel.Style;

import java.net.URL;

import javax.swing.ImageIcon;

/**
 * This class extends java.awt.ImageIcon and provides file name and style property.
 * @author Martin Henke
 *
 */
public class Image extends ImageIcon {

	private static final long serialVersionUID = 3555640635243039425L;
	
	private String fileName = null;
	private Style style = Style.Actual;
	
    /**
     * Creates an uninitialized image icon.
     */
	public Image() {
		super();
	}
	
    /**
     * Creates an ImageIcon from the specified file. The image will
     * be preloaded by using MediaTracker to monitor the loading state
     * of the image. The specified String can be a file name or a
     * file path. When specifying a path, use the Internet-standard
     * forward-slash ("/") as a separator.
     * (The string is converted to an URL, so the forward-slash works
     * on all systems.)
     * For example, specify:
     * <pre>
     *    new ImageIcon("images/myImage.gif") </pre>
     * The description is initialized to the <code>filename</code> string.
     *
     * @param filename a String specifying a filename or path
     * @see #getDescription
     */
	public Image(String fileName) {
		super(fileName);
		this.fileName = fileName;
	}
    /**
     * Creates an ImageIcon from the specified URL. The image will
     * be preloaded by using MediaTracker to monitor the loaded state
     * of the image.
     * The icon's description is initialized to be
     * a string representation of the URL.
     * @param location the URL for the image
     * @see #getDescription
     */
	public Image(URL location) {
		super(location);
	}
    /**
     * Creates an ImageIcon from an image object.
     * If the image has a "comment" property that is a string,
     * then the string is used as the description of this icon.
     * @param image the image
     * @see #getDescription
     * @see java.awt.Image#getProperty
     */
	public Image(java.awt.Image image) {
		super(image);
	}
    /**
     * Creates an ImageIcon from an array of bytes which were
     * read from an image file containing a supported image format,
     * such as GIF, JPEG, or (as of 1.3) PNG.
     * Normally this array is created
     * by reading an image using Class.getResourceAsStream(), but
     * the byte array may also be statically stored in a class.
     * If the resulting image has a "comment" property that is a string,
     * then the string is used as the description of this icon.
     *
     * @param  imageData an array of pixels in an image format supported by
     *             the AWT Toolkit, such as GIF, JPEG, or (as of 1.3) PNG
     * @see    java.awt.Toolkit#createImage
     * @see #getDescription
     * @see java.awt.Image#getProperty
     */
	public Image(byte[] imageData) {
		super(imageData);
	}

    /**
     * Creates an ImageIcon from the specified file. The image will
     * be preloaded by using MediaTracker to monitor the loading state
     * of the image.
     * @param filename the name of the file containing the image
     * @param description a brief textual description of the image
     * @see #ImageIcon(String)
     */
	public Image(String filename, String description) {
		super(filename, description);
	}
    /**
     * Creates an ImageIcon from the specified URL. The image will
     * be preloaded by using MediaTracker to monitor the loaded state
     * of the image.
     * @param location the URL for the image
     * @param description a brief textual description of the image
     * @see #ImageIcon(String)
     */
	public Image(URL location, String description) {
		super(location, description);
	}
    /**
     * Creates an ImageIcon from the image.
     * @param image the image
     * @param description a brief textual description of the image
     */
	public Image(java.awt.Image image, String description) {
		super(image, description);
	}
    /**
     * Creates an ImageIcon from an array of bytes which were
     * read from an image file containing a supported image format,
     * such as GIF, JPEG, or (as of 1.3) PNG.
     * Normally this array is created
     * by reading an image using Class.getResourceAsStream(), but
     * the byte array may also be statically stored in a class.
     *
     * @param  imageData an array of pixels in an image format supported
     *         by the AWT Toolkit, such as GIF, JPEG, or (as of 1.3) PNG
     * @param  description a brief textual description of the image
     * @see    java.awt.Toolkit#createImage
     */
	public Image(byte[] imageData, String description) {
		super(imageData, description);
	}
	
	/**
	 * Gets the file name
	 * @return The file name.
	 */
	public String getFileName(){ return fileName; }
	/**
	 * Sets the file name and loads the specified file.
	 * @param fileName The file name of the file to load.
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
		this.setImage(new ImageIcon(fileName).getImage());
	}
	
	/**
	 * Gets the layout style.
	 * @return The layout style.
	 */
	public Style getStyle(){
		return style;
	}
	/**
	 * Sets the layout style.
	 * @param style The layout style.
	 */
	public void setStyle(Style style){
		this.style = style;
	}
}
