package configuration;

import java.io.Serializable;

import javax.xml.bind.annotation.XmlAttribute;

/**
 * This class Point allows XML serialization. Creation of this class was necessary as the java.awt.Point class creates a Stack Overflow Exception.
 * @author Martin Henke
 *
 */
public class Point implements Serializable  {
	private static final long serialVersionUID = 1184312385090536617L;
	
	/**
	 * The x value.
	 */
	@XmlAttribute(name ="x", required = true)
	public int x;
	/**
	 * The y value.
	 */
	@XmlAttribute(name ="y", required = true)
	public int y;
	
	/**
	 * Initializes the class with x=0 and y=0.
	 */
	public Point() {
		this(0, 0);
	}
	/**
	 * Initializes the class with the specified values.
	 * @param x The x value.
	 * @param y The y value.
	 */
	public Point(int x, int y){
		this.x = x;
		this.y = y;
	}
	
	/**
	 * Initializes the class with the specified java.awt.Point object.
	 * @param point The Point object to provide x and y.
	 */
	public Point(java.awt.Point point){
		this.x = point.x;
		this.y = point.y;
	}
	
	/**
	 * Converts this Point object to a java.awt.Point object.
	 * @return A java.awt.Point object.
	 */
	public java.awt.Point toPoint(){
		return new java.awt.Point(x, y);
	}
}
