package generic;

import java.awt.Graphics;

import javax.swing.ImageIcon;
import javax.swing.JLabel;

import configuration.Image;

/**
 * The class ImageLabel allows to show an image unmodified, streched or tiled.
 * @author Martin Henke
 *
 */
public class ImageLabel extends JLabel {
	private static final long serialVersionUID = -5687863332118337849L;
	
	private Image image = null;
	
	/**
	 * The standard constructor.
	 */
	public ImageLabel() {
		super();
	}
	
	/**
	 * The constructor which allows to set text.
	 * @param text The text the label should display.
	 */
	public ImageLabel(String text){
		super(text);
	}
	
	/**
	 * Gets the Image object.
	 * @return The Image object.
	 */
	public Image getImage(){ return image; }
	/**
	 * Sets the Image object.
	 * @param image The Image object.
	 */
	public void setImage(Image image) {
		this.image = image;
		prepareImage();
	}

	/**
	 * Sets the label size and updates the displayed image.
	 */
	@Override
	public void setSize(java.awt.Dimension dimension){
		super.setSize(dimension);
		prepareImage();
	}
	
	/**
	 * Prepares a temporary Image object and sets it to the super class.
	 */
	private void prepareImage(){
		// Check if there is an image to process
		if (image != null && image.getImage() != null){
			// Create an ImageIcon object which holds the original image.
			ImageIcon imageIcon = new ImageIcon(image.getImage());
			java.awt.Image img;		// Holds the image which will be given to the super class.
			Graphics gr = null;		// The Graphics object for drawing
			switch(image.getStyle()){
			case Actual:
				// No modifications necessary.
				img = imageIcon.getImage();

				// Set the image to the super class
				super.setIcon(new ImageIcon(img));
				break;
			case Scaled:
				// Create an image with the width and height of the label.
				img = createImage(this.getWidth(), this.getHeight());
				
				// Create Graphics object for drawing
				gr = img.getGraphics();
				
				// Draw the scaled image into the new Image object.
				gr.drawImage(img, 0, 0, this.getWidth() - 1, this.getHeight() - 1, 0, 0, imageIcon.getIconWidth() - 1, imageIcon.getIconHeight() - 1, null);
				
				// Tidy up
				gr.dispose();
				
				// Set the image to the super class
				super.setIcon(new ImageIcon(img));
				break;
			case Tiled:
				// Create an image with the width and height of the label.
				img = createImage(this.getWidth(), this.getHeight());
				
				// Create Graphics object for drawing
				gr = img.getGraphics();

				int sx1 = 0;
				int sy1 = 0;
				int sx2 = imageIcon.getIconWidth() - 1;
				int sy2 = imageIcon.getIconWidth() - 1;
				
				// Loop through x and y positions and draw the image at the calculated location
				for (int x = 0; x < this.getWidth(); x = x + imageIcon.getIconWidth()){
					for (int y = 0; y < this.getHeight(); y = y + imageIcon.getIconHeight()){
						gr.drawImage(img, x, y, x + sx2, y + sy2, sx1, sy1, sx2, sy2, null);
					}
				}

				// Tidy up
				gr.dispose();
				
				// Set the image to the super class
				super.setIcon(new ImageIcon(img));
				break;
			default:
				// Clear the icon in the super class
				super.setIcon(null);
				break;
			}
		} else {
			// Clear the icon in the super class
			super.setIcon(null);
		}
	}
}
