package generic;

import java.io.File;
import java.io.FilenameFilter;

import javax.swing.AbstractListModel;
import javax.swing.JList;

import configuration.Skin;

/**
 * The class SkinFileListModel is used by a JList in the SkinLoadDialog (and SkinSaveDialog in SkinEditor) and shows the skin files in the "skins" directory.
 * @author Martin Henke
 *
 */
public class SkinFileListModel extends AbstractListModel<String> {
	
	private static final long serialVersionUID = -7511416413763418033L;

	private String[] files = null;
	
	/**
	 * Creates an instance of the SkinFileListModel and loads the skin files in the "skins" directory into the files array.
	 */
	public SkinFileListModel(){
		File skinDir = new File(Skin.SKIN_DIRECTORY);
		files = null;
		String[] temp = null;
		if (skinDir.exists() && skinDir.isDirectory()){
			temp = skinDir.list(new FilenameFilter(){
				@Override
				public boolean accept(File dir, String name) {
					return (name.toLowerCase().endsWith(Skin.EXTENSION));
				}
				
			});
			if (temp != null){
				files = new String[temp.length];
				String file;
				final int extLength = Skin.EXTENSION.length();
				for (int i = 0; i < temp.length; i++) {
					file = temp[i];
					file = file.substring(0, file.length() - extLength);
					files[i] = file;
				}
			}
		}		
	}
	
	/**
	 * Gets the number of skin files.
	 */
	@Override
	public int getSize() {
		if (files != null){
			return files.length;
		} else {
			return 0;
		}
	}

	/**
	 * Gets the name of the skin file at the specified index.
	 */
	@Override
	public String getElementAt(int index) {
		if (files != null){
			return files[index];
		} else { 
			return null;
		}
	}

	/**
	 * Forces the list to update.
	 * @param source The JList to update.
	 * @param index0 The first index to update.
	 * @param index1 The last index to update.
	 */
	public void update(JList<String>source, int index0, int index1){
		this.fireContentsChanged(source, index0, index1);
	}
}
