package generic;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.EmptyBorder;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

/**
 * The class SkinLoadDialog is similar to the JFileChooser. However, it does not allow to navigate through directories and does not show extensions since only skin
 * files are displayed.
 * @author Martin Henke
 *
 */
public class SkinLoadDialog extends JDialog implements ActionListener, ListSelectionListener, KeyListener {

	private static final long serialVersionUID = 8245380799627417583L;
	
	private static int result = JFileChooser.CANCEL_OPTION;
	
	private JPanel contentPane;
	private JScrollPane scrollPane;
	private JPanel panel;
	private JList<String> lstSkins;
	private JButton btnOK;
	private JButton btnCancel;

	public SkinLoadDialog() {
		setTitle("Select a Skin");
		addKeyListener(this);
		setBounds(100, 100, 450, 300);
		contentPane = new JPanel();
		contentPane.addKeyListener(this);
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);
		
		scrollPane = new JScrollPane();
		scrollPane.addKeyListener(this);
		contentPane.add(scrollPane, BorderLayout.CENTER);
		
		lstSkins = new JList<String>();
		lstSkins.addKeyListener(this);
		lstSkins.addListSelectionListener(this);
		setSkinFileListModel(false);
		scrollPane.setViewportView(lstSkins);
		
		panel = new JPanel();
		panel.addKeyListener(this);
		FlowLayout flowLayout = (FlowLayout) panel.getLayout();
		flowLayout.setAlignment(FlowLayout.RIGHT);
		contentPane.add(panel, BorderLayout.SOUTH);
		
		btnOK = new JButton("Load");
		btnOK.addKeyListener(this);
		btnOK.addActionListener(this);
		btnOK.setEnabled(false);
		btnOK.setPreferredSize(new Dimension(67, 23));
		btnOK.setMinimumSize(new Dimension(67, 23));
		btnOK.setMaximumSize(new Dimension(67, 23));
		panel.add(btnOK);
		
		btnCancel = new JButton("Cancel");
		btnCancel.addKeyListener(this);
		btnCancel.addActionListener(this);
		panel.add(btnCancel);
	}
	
	/**
	 * Creates a SkinFileListModel and updates the list if required.
	 * @param update Defines if the list should be updated.
	 */
	private void setSkinFileListModel(boolean update){
		SkinFileListModel model = new SkinFileListModel();
		lstSkins.setModel(model);
		if (update){
			model.update(this.lstSkins, 0, model.getSize());
		}
	}
	
	/**
	 * Handles clicks.
	 * @param e The associated ActionEvent object.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnCancel) {
			btnCancelActionPerformed(e);
		} else if (e.getSource() == btnOK) {
			btnOKActionPerformed(e);
		}
	}
	/**
	 * Handles the click on btnOK. This sets the result to APPROVE_OPTION and hides the dialog.
	 * @param e The associated ActionEvent object.
	 */
	protected void btnOKActionPerformed(ActionEvent e) {
		result = JFileChooser.APPROVE_OPTION;
		this.setVisible(false);
	}
	/**
	 * Handles the click on btnCancel. This sets the result to CANCEL_OPTION and hides the dialog.
	 * @param e The associated ActionEvent object.
	 */
	protected void btnCancelActionPerformed(ActionEvent e) {
		result = JFileChooser.CANCEL_OPTION;
		this.setVisible(false);
	}
	
	/**
	 * Returns the name of the selected file or null.
	 * @return The name of the selected file or null.
	 */
	public String getSelectedFile(){
		if (lstSkins.getSelectedIndex() > -1){
			return lstSkins.getSelectedValue();
		} else {
			return null;
		}
	}
	/**
	 * Handles value changes in the list.
	 * @param e The associated ListSelectionEvent object.
	 */
	public void valueChanged(ListSelectionEvent e) {
		if (e.getSource() == lstSkins) {
			lstSkinsValueChanged(e);
		}
	}
	/**
	 * Handles value changes in the list lstSkins. This enables the button btnOK if an entry is selected, otherwise disables it.
	 * @param e The associated ListSelectionEvent object.
	 */
	protected void lstSkinsValueChanged(ListSelectionEvent e) {
		this.btnOK.setEnabled(lstSkins.getSelectedIndex() > -1);
	}
	
	/**
	 * Shows the dialog and returns the result of selection (APPROVE_OPTION or CANCEL_OPTION).
	 * @return The result of selection (APPROVE_OPTION or CANCEL_OPTION)
	 */
	public int showDialog(){
		this.setModal(true);
		this.setVisible(true);
		return result;
	}
	/**
	 * Currently not used.
	 */
	public void keyPressed(KeyEvent e) { }
	/**
	 * Currently not used.
	 */
	public void keyReleased(KeyEvent e) { }
	/**
	 * Handles key presses of the user. Updates the list if the user presses the F5 key.
	 * @param e The associated KeyEvent object.
	 */
	public void keyTyped(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_F5){
			setSkinFileListModel(true);
		}
	}
}
